<?php
/**
 * $Id: js_res_file_manager.php 7311 2011-08-19 11:18:02Z shitz $
 * $LastChangedDate: 2011-08-19 17:18:02 +0600 (Fri, 19 Aug 2011) $
 * $LastChangedBy: shitz $
 * MightyResources by Mighty Extensions
 * a component for Joomla! 1.5 CMS (http://www.joomla.org)
 * Author Website: http://www.mightyextensions.com/
 * @copyright Copyright (C) 2006 - 2011 MightyExtensions (http://www.mightyextensions.com). All rights reserved.
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
*/
defined('_JEXEC') or die('Restricted access');


/**
 * Class that helps to utilize new filesystem layout: [userid/extension/file]
 * Note: uses singletone pattern %))
 * @author Nurlan
 *
 */
class JS_Res_File_Manager extends JObject
{
	/**
	 * File type filter.
	 * @var array
	 */
	var $_filter = null;
	
	/**
	 * If set to true, all the URLs to files will be masked like "index2.php?file=blahblah"
	 * @var boolean
	 */
	var $_hideURL = true;
	
	var $_allowedExtensions = null;
	
	/**
	 * The permission for folders
	 * @var int
	 */
	var $folderMode = 0755;
	
	/**
	 * The permission for files
	 * @var int
	 */
	var $fileMode = 0644;
	
	/**
	 * Fetches filemanager instance 
	 * @return JS_Res_File_Manager Filemanager instance
	 */
	function & getInstance() {
		static $instance = 0;
		if (!$instance) {
			$instance = new JS_Res_File_Manager();
		}
		return $instance;
	}
	
	/**
	 * Constructor. Warning: do not use it directly, use JS_Res_File_Manager::getInstance instead
	 * @return unknown_type
	 */
	function JS_Res_File_Manager() {
		parent::__construct();
		
		$this->_initResourceConfig();
		
		$this->_filter = array();
		
		$this->_filter['image'] = array('jpg', 'jpeg', 'bmp', 'gif', 'png', 'svg', 'tiff');
		$this->_filter['video'] = array("swf", "avi", "mov", "wmv", "mpeg", "mpg", "mpe", 'flv', 'mp4');
		$this->_filter['mp3'] = array("mp3", 'mpg3');
		
		// Route all URLs
		$this->set('routeURL', true);
		
		// Mask all URL's by default.
		$this->_hideURL = true;
		
		$this->disallowExtension(array('php', 'ini'));
	}
	
	/**
	 * Set, whether to mask the URLs
	 * @param $flag boolean
	 * @return boolean previous value
	 */
	function setUrlsMasked($flag) {
		$old = $this->_hideURL;
		$this->_hideURL = !!$flag;
		return $old;
	}
	
	/**
	 * See whether URL's are being masked
	 * @return boolean
	 */
	function isUrlsMasked() {
		return $this->_hideURL;
	}
	/**
	 * Get a URL for a file
	 * @param $file
	 * @param $ownerId
	 * @param $fieldValueId
	 * @param $displayName
	 * @return unknown_type
	 */
	function getDownloadUrl($file, $ownerId, $fieldValueId = 0, $displayName = '') {
		// user id may be 0
		if (!$file || is_null($ownerId))
			return false;
		
		$fieldValueId 	= (int) $fieldValueId;
		$ownerId 		= (int) $ownerId;
		
		$ext = strtolower(JFile::getExt($file));
		if ($ext == 'jpeg')
			$ext = 'jpg';
		$file = str_replace(RES_GENERAL_UPLOAD.DS.$ownerId.DS.$ext.DS, '', $file);
		
		// Mask URLs
		if ($this->_hideURL) {
			$url = 'index2.php?option=com_resource&no_html=1&task=show_file&file='.base64_encode($file).'&uid='.$ownerId;
			if ($fieldValueId)
				$url .= '&id='.$fieldValueId;
			if ($displayName)
				$url .= '&name='.base64_encode($displayName);
			
			if ($this->get('routeURL'))
				$url = JRoute::_($url);
			
			if ($this->get('encodeURL'))
				$url = rawurlencode($url);
			
			// A Hack: type added after the url is encoded [to use with shadowbox video player]
			if ($this->get('type'))
				$url .= '&type='.rawurlencode($this->get('type'));
				
			return $url;
		}
		
		// Direct link to UPLOADS dir
		$folder = RES_GENERAL_UPLOAD;
		$folder = str_replace(array(JPATH_ROOT.DS, str_replace('\\', '/',JPATH_ROOT.DS)), '', $folder);
		$folder = str_replace('\\','/', $folder);
		
		return $folder.'/'.$ownerId.'/'.$ext.'/'.$file;
	}
	
	/**
	 * Identifies file type
	 * @param $ext
	 * @return mixed
	 */
	function resolveFileType($ext) {
		$ext = strtolower($ext);
		foreach ($this->_filter as $type => $extensions) {
			if (in_array($ext, $extensions))
				return $type;
		}
		
		return null;
	}
	
	/**
	 * Returns file types, and extensions assigned to them.
	 * @param $typesOnly boolean
	 * @return array
	 */
	function getFileTypes($typesOnly = true) {
		if ($typesOnly)
			return array_keys($this->_filter);
		else
			return $this->_filter;
	}
	
	/**
	 * Assigns extension to file types.
	 * @param $type
	 * @param $extension
	 * @return unknown_type
	 */
	function addFileType($type, $extension) {
		if (!isset($this->_filter[ $type ]))
			$this->_filter[ $type ] = array();
			
		if (!in_array($extension, $this->_filter[ $type ]))
			$this->_filter[ $type ][] = $extension;
	}
	
	
	/**
	 * Copies <b>newly</b> uploaded file to it's predefined place
	 * 
	 * @param $filename string The file name
	 * @param $destName string The destination file name
	 * @param $userId int The owner of the file
	 * @param $dir string The nested directory
	 * @param $move boolean
	 * @return mixed Saved file name, null on bad args.
	 */
	function storeFile($filename, $destName, $userId, $dir = '', $move = false) {
		// userId may be 0
		if (!$filename || is_null($userId))
			return null;
			
		if (!$destName)
			$destName = $filename;
			
		$userId = (int)$userId;
		
		$inFile = $this->getPath($filename, $userId);
		$destFile = $this->getPath($destName, $userId, $dir);
		
		if (!$inFile || !$destFile)
		{
			return false;
		}
		
		// Make sure these files are allowed to use
		if ( !$this->isAllowedExtension(JFile::getExt($inFile)) )
			return false; 
		if ( !$this->isAllowedExtension(JFile::getExt($destFile)) )
			return false; 
		
		if ($inFile == $destFile)
			return $destFile;

		if (JFile::exists($inFile))
		{
			$folder = dirname($destFile);
			$this->_createSecureFolder($folder);
			
			if (!JFile::copy($inFile, $destFile)) {
				JError::raiseError(JText::_('Cannot copy').' <b>'.$inFile.'</b> '.JText::_('to').' <b> '.$destFile.'</b>');
			}
			if ($move)
				JFile::delete($inFile);
			// set the file permissions	
			$this->_setFilePerms($destFile);
			
			return $destFile;
		}
		
		return false;
	}
	
	/**
	 * Safely delete file
	 * @param $filename
	 * @return boolean the result
	 */
	function deleteFile($filename) 
	{	
		if (JFile::exists($filename) && $this->checkPath($filename))
			return JFile::delete($filename);
		
		return false;
	}
	
	
	/**
	 * Returns file's path
	 * @param $file
	 * @param $userId
	 * @param $dir
	 * @return unknown_type
	 */
	function getPath($file, $userId, $dir = '') {
		$file = JFile::makeSafe($file);
		$userId = (int)$userId;
		
		$ext = strtolower(JFile::getExt( $file ));
		if ($ext == 'jpeg') {
			$ext = 'jpg';
			$file = JFile::stripExt($file).'.'.$ext;
		}
		
		$ret = RES_GENERAL_UPLOAD.DS.$userId.DS.$ext;
		if ($dir)
			$ret .= DS.$dir;
		$ret .= DS. $file;
		
		if (!$this->checkPath($ret))
		{
			return false;
		}
		
		return $ret;
	}
	
	/**
	 * Check the file path
	 * @param $file
	 * @return unknown_type
	 */
	function checkPath($file) 
	{
		// check for bad pathes
		if (strpos($file, '..') !== false) 
		{
			$this->setError(JText::_('Use of relative paths not permitted'));
			return false;
		}
		// check for bad pathes
		$file = JPath::clean($file);
		if (strpos($file, JPath::clean(JPATH_ROOT)) !== 0) 
		{
			$this->setError(JText::_('Snooping out of bounds'));
			return false;
		}
		
		// Check that file is in RES_GENERAL_UPLOAD directory
		if (strpos($file, JPath::clean(RES_GENERAL_UPLOAD)) === false)
		{
			return false;
		}
		
		return true;
	}
	
	/**
	 * Assigns files to new user.
	 * @param $files array
	 * @param $articleId string
	 * @param $prevAuthor int
	 * @param $currentAuthor int
	 * @param $directory string
	 * @return unknown_type
	 */
	function reassignFiles($files, $articleId, $prevAuthor, $newAuthor, $directory = '') {
		jimport('joomla.filesystem.file');
		jimport('joomla.filesystem.folder');
		
		settype($files, 'array');
		$prevAuthor 	= (int)$prevAuthor;
		$newAuthor 	= (int)$newAuthor;
		
		$folder = RES_GENERAL_UPLOAD;
		
		foreach($files as $f)
		{
			if (!$f)
				continue;
			
			$f = basename( $f );
			$ext = JString::strtolower(JFile::getExt( $f ));
			if ($ext == 'jpeg')
				$ext = 'jpg';
			
			$f = JFile::makeSafe(JFile::stripExt( $f ).'.'.$ext);
			
			$infile = JPath::clean($folder.DS.$prevAuthor.DS.$ext.DS.$directory.DS. $articleId.'_'.$f);
			
			$destFile = JPath::clean($folder.DS.$newAuthor.DS.$ext.DS .$directory.DS.$articleId.'_'.$f);
			
			$dir = dirname($destFile);
			$this->_createSecureFolder($dir);

			if (JFile::exists($infile))
			{
				if (JFile::exists($destFile))
					JFile::delete($destFile);
				
				JFile::copy($infile, $destFile);
				JFile::delete($infile);
				
				// set the file permissions	
				$this->_setFilePerms($destFile);
			}
		}
	}
	
	/**
	 * Donwloads a file
	 * 
	 * @param $filename
	 * @param $displayName
	 * @param $userId
	 * @param $fieldValueId
	 * @return unknown_type
	 */
	function downloadFile($filename, $displayName, $userId, $fieldValueId = 0, $type = false) {
		$userId 	= (int)$userId;
		
		$ext = strtolower(JFile::getExt($filename));
		// $file = JPath::clean(RES_GENERAL_UPLOAD.DS.$userId.DS.$ext.DS.$filename);
		
		$file = $this->_resolveFileName($userId, $filename, $type);
		
//		echo '<pre>';
//		var_dump($file, $this->resolveFileType($ext));
//		die;
		
		// Check file download policy
		if ( !$this->isAllowedExtension($ext) )
			return false;
			
		$fileId = (int)$this->get('file_id');
		if ($fileId) {
			$db = & JFactory::getDBO();
			
			$db->setQuery('SELECT fe.value FROM #__js_res_field_extra fe WHERE ref_id = "file" AND fe.id = '.$fileId.' AND user_id = '.$userId);
			$file = $db->loadResult();
			$ext = strtolower(JFile::getExt($file));
			$filename = $file;
		}
		
		if(JFile::exists($file))
		{
			// Track the download:
			$recordId 	= (int)$this->get('record_id');
			$fieldId 	= (int)$this->get('field_id');
			$fvId 		= (int)$this->get('id');
			
			if ($recordId && $fieldId && $fvId) {
				ResController::hit2($recordId, $fieldId, $fvId, basename($displayName));
			}
			
			$fileType = $this->resolveFileType($ext);
			
			if ($fileType == 'image')
				return $this->_downloadImage($file, $displayName);
		
			if ($fileType == 'video') {
				return $this->_downloadVideo($file, $displayName);
			}
			
			if ($fileType == 'mp3')
				return $this->_downloadMp3($file, $displayName);
			
			if (!$fileId)
			{
				// now, make the filename safe, do not allow directory names in filename.
				$filename 	= JFile::makeSafe($filename);
				// The extension may become a little different after JFile::makeSafe
				$ext 		= strtolower(JFile::getExt($filename));
				$file 		= JFile::stripExt($filename).'.'.JFile::getExt($filename);
			}		
		}
		if ($fieldValueId >= 0)
			return $this->_downloadFile($filename, $displayName, $fieldValueId);
		
		return false;
	}
	
	/**
	 * Resolves filename on the disk
	 * @param $userId
	 * @param $filename
	 * @param $type string the file type
	 * @return string filename
	 */
	function _resolveFileName($userId, $filename, $type = false) {
		$userId 	= (int)$userId;
		
		$ext = strtolower(JFile::getExt($filename));
		if ($type)
			return JPath::clean(RES_GENERAL_UPLOAD.DS.$type.DS.$userId.DS.$filename);
		else
			return JPath::clean(RES_GENERAL_UPLOAD.DS.$userId.DS.$ext.DS.$filename);
	}
	
	function _downloadImage($file) {
		jimport ('joomla.filesystem.file');
                /*echo $file;
                exit;*/
		if(JFile::exists($file))
		{
			$imageInfo = @getimagesize($file);
			
			if (is_array($imageInfo) && $mimeType = $imageInfo['mime'])
			{	
				@ob_end_clean();
				header("Content-Type: ".$mimeType);
				header('Content-disposition: inline; filename='.basename($file));
				header('Expires: '.date('D, d M Y H:i:s', time() + 86400).' GMT');
				header('Pragma: private');
	
				JS_Res_File_Manager::readFileChunked($file);
				exit;
			}
		}
		return false;
	}
	
	/**
	 * Donwload a file: 
	 * 		if $this->get('file_id') is given, looks in #__js_res_field_extra for the filename.
	 * 
	 * Uses $this->get('record_id'), $this->get('field_id'), $this->get('id') for hit tracking
	 * @param $file
	 * @param $downloadName
	 * @return unknown_type
	 */
	function _downloadFile($file) {
		$browser =  $_SERVER['HTTP_USER_AGENT'];

		if(ereg('Opera(/| )([0-9].[0-9]{1,2})', $browser)) {
			$browser =  'OPERA';
		} else if(ereg('MSIE ([0-9].[0-9]{1,2})', $browser)) {
			$browser =  'IE';
		} else if(ereg('OmniWeb/([0-9].[0-9]{1,2})', $browser)) {
			$browser =  'OMNIWEB';
		} else if(ereg('(Konqueror/)(.*)', $browser)) {
			$browser =  'KONQUEROR';
		} else if(ereg('Mozilla/([0-9].[0-9]{1,2})', $browser)) {
			$browser =  'MOZILLA';
		} else {
			$browser =  'OTHER';
		}
		
		if (!$downloadName)
			$downloadName = basename($file);
		
		// Security check.
		JPath::check($file);

		if (JFile::exists($file))
		{
			// Prevent inclusion of Xajax scripts, and other stuff...
			@ob_end_clean();

			header('Content-Type: '.(($browser=='IE' || $browser=='OPERA')?
			'application/octetstream':'application/octet-stream'));
			header('Expires: '.gmdate('D, d M Y H:i:s').' GMT');
			header('Content-Transfer-Encoding: binary');
			header('Content-Length: '.filesize($file));
			header("Content-Encoding: none");
			if($browser=='IE') {
				header('Content-Disposition: attachment; filename="'.basename($file).'"');
				header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
				header('Pragma: public');
			} else {
				header('Content-Disposition: attachment; filename="'.basename($file).'"');
				header('Cache-Control: no-cache, must-revalidate');
				header('Pragma: no-cache');
			}

			JS_Res_File_Manager::readFileChunked($file);
		}
		return false;
	}
	
	/**
	 * Behavior affected by $this->get('showPlaylist') && $this->get('uid')
	 * 
	 * @param $file string video file to download
	 * @return unknown_type
	 */
	function _downloadVideo($file)
	{	
		$ext = strtolower(JFile::getExt($file));

		switch ($ext)
		{
			case "swf":
				$type = 'application/x-shockwave-flash';
				break;
			case "avi":
				$type = 'video/x-msvideo ';
				break;
			case "mov":
				$type = 'video/quicktime';
				break;
			case "wmv":
				$type = 'video/x-ms-wmv';
				break;
			case "mpeg":
			case "mpg":
			case "mpe":
				$type = 'video/mpeg';
			case 'mp4':
				$type = 'video/mp4';
			case 'flv':
				$type = 'video/x-flv';
		}
		
		if ($this->get('showPlaylist') && $this->get('uid'))
		{
			@ob_end_clean();
			
			require_once JPATH_PLUGINS.DS.'joomsuite_content_fields'.DS.'html'.DS.'uploader.php';
			
			$needle = JPath::clean(RES_GENERAL_UPLOAD.DS.(int)$this->get('uid').DS.strtolower(JFile::getExt($file)).DS);
			
			$file = str_replace($needle, '', $file);
			
			$url = 'index2.php?option=com_resource&no_html=1&task=show_file&file='.base64_encode($file).'&uid='.(int)$this->get('uid');
			$info = new stdClass;
			$info->file = $info->title = basename($file);
			$info->link = JURI::root();
			
			$url = JFilterOutput::ampReplace(JURI::root().$url);

			// Generate JWPlayer RSS playlist
			echo JSJWPlaylistGenerator::generateRSSForURL($url, $info);
			exit;
		}
		
		if(JFile::exists($file))
		{
			@ob_end_clean();
			
			header("Content-Type: $type");
			header("Content-Length: " . filesize($file));
			header('Content-disposition: attachment; filename='.basename($file));
			
			JS_Res_File_Manager::readFileChunked($file);
			exit;
		}
		return false;
	}
	
	function _downloadMp3($file) {
		if (JFile::exists($file))
		{
			header("Content-Type: audio/mpeg");
			header('Content-disposition: attachment; filename='.basename($file));
			header("Content-Length: " . filesize($file));
			
			JS_Res_File_Manager::readFileChunked($file);
			exit;
		}
		return false;
	}
	
	/**
	 * Checks if the extension is allowed to download
	 * @param $ext string Extension to check
	 * @return unknown_type
	 */
	function isAllowedExtension($ext) {
		if ( isset($this->_allowedExtensions[$ext]) ) {
			return $this->_allowedExtensions[$ext];
		}
		return true;
	}
	
	/**
	 * Add extension to allowed extension list
	 * @param $exts mixed extension
	 * @return void
	 */
	function allowExtension($exts) {
		settype($exts, 'array');
		
		foreach ($exts as $ext) {
			$ext = JFile::makeSafe($ext);
			// avoid dots in extension
			if (strpos($ext, '.') !== false)
				$ext = JFile::getExt($ext);
			
			$this->_allowedExtensions [$ext] = true;
		}
	}
	
	/**
	 * Dissallow extension
	 * @param $exts mixed Extension
	 * @return void
	 */
	function disallowExtension($exts) {
		settype($exts, 'array');
		
		foreach ($exts as $ext) {
			$ext = JFile::makeSafe($ext);
			// avoid dots in extension
			if (strpos($ext, '.') !== false)
				$ext = JFile::getExt($ext);
			
			$this->_allowedExtensions[$ext] = false;
		}
	}
	
	function _createSecureFolder($folder) {
		$mode = $this->folderMode ? $this->folderMode :  0755;
		
		if (!JFolder::exists($folder))
			return JFolder::create($folder, $mode);
		else
			return JPath::canChmod($folder) && @chmod($folder, $mode); 
	}
	
	function _setFilePerms($file) {
		$mode = $this->fileMode ? $this->fileMode :  0644;
		
		if (JFile::exists($file)) {
			JPath::canChmod($file) && @chmod($file, $mode);
		}
	}
	/**
	 * Initializes com_resource settings 
	 * @return void
	 */
	function _initResourceConfig() {
		static $isInitialized = 0;
		
		if (!$isInitialized)
		{
			jimport ('joomla.filesystem.file');
			jimport ('joomla.filesystem.folder');
			
			require_once (JPATH_ROOT.DS.'administrator'.DS.'components'.DS.'com_resource'.DS.'controllers'.DS.'config.php');
			ResControllerConfig::initialyze();
			
			// Include the com_resource main controller to access it's methods
			require_once JPATH_ROOT.DS.'components'.DS.'com_resource'.DS.'controller.php';
			
			$isInitialized = 1;
		}
	}
        /**
         * Method that reads file by chunks and returns the data
         * @param string $filename
         * @param boolean $retbytes
         * @return if $retbytes is true will return the number of bytes read, otherwise will return the status
         */
        function readFileChunked($filename,$retbytes=true) {
		error_reporting(0);
		//ignore_user_abort(true);
		set_time_limit(0);
		$chunksize = 1*(1024*1024); // how many bytes per chunk
		$buffer = '';
		$cnt =0;
		// $handle = fopen($filename, 'rb');
		$handle = fopen($filename, 'rb');
		if ($handle === false) {
			echo "Cannot open file"; exit;
			return false;
		}
		while (!feof($handle)) {
			$buffer = @fread($handle, $chunksize);
			echo $buffer;
			if ($retbytes) {
				$cnt += strlen($buffer);
			}
		}
		$status = fclose($handle);
		if ($retbytes && $status) {
			return $cnt; // return num. bytes delivered like readfile() does.
		}
		return $status;
	}
}

?>