var L = {
	debug: false,
	
	log: function( s ) {
		if (L.debug) {
			if (window.console)
				console.log( s );
			else
				alert( s );
		}
	}
}

var JS_IFrame_Uploader = function(varname) {
	this.varname = varname;
	this.validators = [];
	this.forms = [];
	this.listeners = [];
}

JS_IFrame_Uploader.prototype = {
	strings: {
		UNKNOWN_ERROR: 'Error'
	},
	varname: null,
	forms: [],
	listeners: [],
	validators: [],
	
	addForm: function( f ) {
		for (var i = 0; i < this.validators.length; i++) {
			if (!this.validators[ i ].validateCount(this.forms.length + 1)) {
				this.triggerEvent('onFileLimit', [this.validators[ i ].getMax()]);
				return false;
			}
		}
	
		if (!f.target) {
			f.target = this.createIFrame();
		}

		document.getElementById(f.target).targetForm = f;

		ifdoc = this._getIframeDoc(f.target);
		ifdoc.targetForm = f;

		L.log('addForm: form and iframe');
		L.log( document.getElementById(f.target) );
		L.log( ifdoc );
		
		this.forms.push( f );
		this.triggerEvent('onFormAdd', [this.forms, f]);
	},
	
	popForm: function(idx) {
		if (idx == undefined)
			idx = this.forms.length - 1;
		if (idx < 0)
			idx = 0;

		// handle idx being a form
		if (idx != undefined && !(idx instanceof String)) {
			L.log('form object');
			var i = 0, found = false;
			for (i = 0; i < this.forms.length; i++) {
				if (idx == this.forms[ i ]) {
					found = true;
					break;
				}
			}

			if (found)
				idx = i;

			L.log('index ' + idx);
		}
		
		var f = this.forms[ idx ];
		if ( f ) {
			this.forms = this.forms.slice(0, idx).concat(this.forms.slice(idx + 1));
			return f;
		}
		return false;
	},
	
	uploadForms: function(form, iframe) {
		fs = this.forms;
		
		this.triggerEvent('onBeforeSubmit');
		
		for (var i = 0; i < fs.length; i++) 
		{
			this.triggerEvent('onSubmit', [fs[ i ]]);
			
			var res = this._uploadForm( fs[ i ] );
			if (res != undefined && res != true) {
				this.triggerEvent('onUploadFail', [fs[ i ], res]);
			}
		}
	},
	
	createIFrame: function() {
		var d = document.createElement('DIV');
		
		// Set unique id
		var n = null;
		do {
			n = 'up' + Math.floor(Math.random() * 9999999);
		} while (document.getElementById( n ));
		
		d.innerHTML = '<iframe style="display:none" src="about:blank" id="' + n + '" name="' + n + 
			'" onload="' + this.varname + '.handleIframe.call(' + this.varname + ', this)' + '"></iframe>';
		d.id = n + '_outer';
		document.body.appendChild( d );
		
		return n;
	},

	handleIframe: function( iframe ) {
		var ifdoc = this._getIframeDoc(iframe.id);
		var form = iframe.targetForm;


		L.log('handleIframe: Frame href = ' + ifdoc.location.href);
		
		if (ifdoc.location.href == 'about:blank' || !form)
			return;
		
		var json = ifdoc.body.innerHTML;
		var file = form.elements[ 0 ];

		var response = null;
		try {
			response = eval('(' + json + ')');
		}
		catch ( e ) {
			response = {error: this.strings.UNKNOWN_ERROR}
		}

		// Response data
		var data = {
			container: form.parentNode,
			file: file,
			response: response
		};
		
		this.triggerEvent('onAfterFileUpload', [form, iframe, data]);

		// unregister the form
		if (form)
			this.popForm(form);
	},

	addValidator: function(val) {
		if (!this.validators)
			this.validators = [];
		this.validators.push( val );
	},
	
	_uploadForm: function( f ) {
		if (!this.validators)
			return true;

		if (!f.elements[ 0 ].value)
			return [{type: 'empty'}];

		var errors = [];
		
		for (var i = 0; i < this.validators.length; i++) {
			var test = this.validators[ i ].validateExt(f.elements[ 0 ]);
			if (test != true)
				errors.push(test);
		}

		if (errors.length)
			return errors;
		
		var filename = f.elements[ 0 ].value;
		var idx = filename.lastIndexOf('\\');
		if (idx >= 0)
			filename = filename.substr(idx + 1); 
		
		f.action += '&Filename=' + encodeURIComponent(filename);
		
		L.log( f );
		
		return f.submit();
	},
	/*******************************************************************************/
	triggerEvent: function(event, args) {
		if (!this.listeners[ event ])
			return null;

		var results = [];
			
		var ev = this.listeners[ event ];
		for (var i = 0; i < ev.length; i++) {
			results.push( ev[ i ].f.apply(ev[ i ].o, args) );
		}
		
		L.log('Event: ' + event);

		return results;
	},
	
	addListener: function(event, func, obj) {
		if (!this.listeners[ event ])
			this.listeners[ event ] = [];
		this.listeners[ event ].push({f: func, o: obj});
	},
	
	removeListener: function(event, func) {
		var e = this.listeners[ event ];
		if (!e)
			return;
		
		for (var i = 0; i < e.length; i++) {
			if (e[ i ].f && e[ i ].f == func) {
				e[ i ] = e.slice(0, i - 1).concat(e.slice(i + 1));
				break;
			}
		}
	},
	
	_getIFrameAsJson: function( id ) {
		var d = this._getIframeDoc( id );
		if (d.location.href == "about:blank") {
			return;
		}
		
		return eval('(' + d.body.innerHTML + ')');
	},

	_getIframeDoc: function( id ) {
		var i = document.getElementById( id );
		
		if (i.contentDocument) {
			var d = i.contentDocument;
		} else if (i.contentWindow) {
			var d = i.contentWindow.document;
		} else {
			var d = window.frames[id].document;
		}

		return d;
	}
}

var UploaderDomHelper = function(uploader, filelist, filename, action, img) {
	this.uploader = uploader;
	this.filelist = filelist;
	this.filename = filename;
	this.loadingImg = img;
	this.action = action;
	
	this.files = {};
}

UploaderDomHelper.prototype = {
	strings: {
		UNKNOWN_ERROR: 'Error',
		FILE_LIMIT_ERROR: 'Maximum allowed files: %d',
		REMOVE: 'Remove',
		FILE: 'File'
	},
	uploader: null,
	filelist: null,
	filename: null,
	loadingImg: null,
	action: null,
	
	watcher: null,
	
	files: [],
	
	/// event to attach on <a> elements to initiate file deletion
	onDeleteFile: null,
	
	createNewItem: function() {
		var f = this._createElement('form', {
			action: this.action,
			method: 'post',
			enctype: 'multipart/form-data',
			encoding: 'multipart/form-data'
		});
		
		var file = this._createElement('input', {
			name: 'upload',
			type: 'file',
			className: 'inputbox'
		});
		
		
		var label = this._createElement('label', {
			innerHTML: this.strings.FILE + ': '
		});
		
		var p = this._createElement('P');
		p.style.margin = 0;

		var span1 = this._createElement('span', {innerHTML: '[ + ]'});
		var span2 = this._createElement('span', {innerHTML: '[ - ]'});

		span1.style.cursor = span2.style.cursor = 'pointer';

		span1.domHelper = this;
		span1.targetForm = f;
		span1.uploader = this.uploader;

		span2.domHelper = this;
		span2.targetForm = f;
		span2.uploader = this.uploader;

		// add new form
		span1.onclick = function() {
			var el = this.domHelper.createNewItem();
			this.uploader.addForm(el.firstChild);
		}

		span2.onclick = function() {
			var f = this.targetForm;
			var p = f.parentNode;				
			this.uploader.popForm( f );

			p.parentNode.removeChild( p );
		}
		
		label.appendChild( file );
		f.appendChild( span1 );
		f.appendChild( span2 );
		f.appendChild( label );
		p.appendChild( f );

		f.containerReference = p;
		
		return p;
	},

	createResultItem: function(result) {
		var text;
		var container = result.container;
		var className;

		L.log(result);

		if (result.response.error) {
			text = result.file.value + ' : ' + result.response.error;
			className = 'error';
		}
		else {
			text = result.response.size;
			className = 'success';
		}

		var span = this._createElement('span', {className: className, innerHTML: text});
		container.appendChild( span );
		
		if (this.watcher && this.watcher.createResultItem)
			this.watcher.createResultItem( span );
	},

	addToFileList: function(filename) {
		var idx = filename.lastIndexOf('\\');
		if (idx >= 0)
			filename = filename.substr(idx + 1); 
		
		L.log('AddToFileLIst ' + filename);
		if (this.files) {
			if (this.files[ filename ]) {
				L.log('false');
				return false;
			}
		}
		
		this.files[ filename ] = true;
		
		L.log(this.files);
		
		var tr = this._createElement('tr');
		tr.appendChild(this._createElement('td', {align: 'left', innerHTML: filename + ' '}));

		var td = this._createElement('td');
		
		var deleteFunction = this.onDeleteFile;
		
		var a = this._createElement('a', {
			href: 'javascript:void(0);',
			innerHTML: this.strings.REMOVE,
			domHelper: this,
			filename: filename,
			'onclick': function() {
				deleteFunction( a );
			}
		});
		
		td.appendChild( a );

		td.appendChild(this._createElement('input', {
			type: 'hidden',
			name: this.filename,
			value: filename
		}));		

		tr.appendChild( td );
		
		this.filelist.appendChild( tr );
		
		return true;
	},
	
	removeFromFileList: function(row, filename) {
		L.log('removeFromFileList: ' + filename);
		L.log(this.files);
		if (this.files[ filename ]) {
			var vals = this.uploader.validators;
			for (var i in vals) {
				if (vals[ i ].incrementFileCount)
					vals[ i ].incrementFileCount( -1 );
			}
			
			if (row) {
				this.files[filename] = false;
				row.parentNode.removeChild( row );
			}
		}
	},
	
	onSubmit: function(form) {
		L.log('onSubmit domHelper');
		var p = form.parentNode;
		
		var e = this._createElement('img', {
			className: 'removeMeNextTime',
			src: this.loadingImg,
			visible: true,
			display: 'block'
		});
		L.log( e );
		p.appendChild( e );

		form.style.display = 'none';
	},
	
	onAfterFileUpload: function(form, frame, data) {
		if (form == undefined || !data) {
			L.log('onAfterFileUpload: form or data is undefined');
			return;
		}

		L.log('onAfterFileUpload');
		this._removeControls(form, frame);
		
		this.createResultItem(data);
		if (!data.response.error) {
			// If the file is on the list:
			if (!this.addToFileList(form.elements[ 0 ].value)) {
				this.removeFromFileList(null, form.elements[ 0 ].value);
			}
		}
	},	

	onUploadFail: function(form, errors) {
		if (errors) {
			var text = '';
			for (var i = 0; i < errors.length; i++) {
				if (errors[ i ].message)
					text += errors[ i ].message + '<br />';
			}

			for (var i = 0; i < errors.length; i++) {
				if (errors[ i ] && errors[ i ].type && errors[ i ].type == 'empty') {
					L.log('onUploadFail empty form');
				}
				else if (errors[ i ] && errors[ i ].type && errors[ i ].type == 'bad_extension') {
					L.log('Bad extension');
					alert( errors[ 0 ].message );
				}
				else {
					form.parentNode.appendChild(this._createElement('span', {
						innerHTML: text,
						className: 'uploader_failed'
					}));
				}
			}
			
			this.uploader.popForm(form);
			var p = form.parentNode;
			p.parentNode.removeChild( p );
		}
	},

	onFileLimit: function( max ) {
		alert(this.strings.FILE_LIMIT_ERROR.replace('%d', max));
	},

	_createElement: function(type, attr) {
		var f = document.createElement(type.toUpperCase());
		for (var i in attr)
			f[ i ] = attr[ i ];
		return f;
	},

	_removeControls: function(form, frame) {
		if (frame && /msie/i.test(navigator.userAgent) && !/opera/i.test(navigator.userAgent)) {
			// remove the iframe with it's container, if we are in IE
			e = document.getElementById(frame.id + '_outer');
			e.parentNode.removeChild( e );
		}

		if (form) {
			try {
				// Empty container:
				var p = form.parentNode;
				p.removeChild( form.nextSibling );
				p.removeChild( form );
			}
			catch ( e ) {
			}
		}
	}
}

var UploaderDomHelperWithTitle = function(uploader, filelist, filename, action, img) {
	this.uploader = uploader;
	this.filelist = filelist;
	this.filename = filename;
	this.loadingImg = img;
	this.action = action;
	
	this.files = {};
}

UploaderDomHelperWithTitle.prototype = {
		strings: {
			UNKNOWN_ERROR: 'Error',
			FILE_LIMIT_ERROR: 'Maximum allowed files: %d',
			REMOVE: 'Remove',
			FILE: 'File',
			TITLE: 'Title',
			UNTITLED: 'Untitled'
		},
		uploader: null,
		filelist: null,
		filename: null,
		loadingImg: null,
		action: null,
		
		watcher: null,
		
		files: [],
		
		/// event to attach on <a> elements to initiate file deletion
		onDeleteFile: null,
		
		createNewItem: function() {
			var f = this._createElement('form', {
				action: this.action,
				method: 'post',
				enctype: 'multipart/form-data',
				encoding: 'multipart/form-data'
			});
			
			var file = this._createElement('input', {
				name: 'upload',
				type: 'file',
				className: 'inputbox'
			});
			
			
			var label = this._createElement('label', {
				innerHTML: this.strings.FILE + ': '
			});
			
			var p = this._createElement('P');
			p.style.margin = 0;

			var span1 = this._createElement('span', {innerHTML: '[ + ]'});
			var span2 = this._createElement('span', {innerHTML: '[ - ]'});

			span1.style.cursor = span2.style.cursor = 'pointer';

			span1.domHelper = this;
			span1.targetForm = f;
			span1.uploader = this.uploader;

			span2.domHelper = this;
			span2.targetForm = f;
			span2.uploader = this.uploader;

			// add new form
			span1.onclick = function() {
				var el = this.domHelper.createNewItem();
				this.uploader.addForm(el.firstChild);
			}

			span2.onclick = function() {
				var f = this.targetForm;
				var p = f.parentNode;				
				this.uploader.popForm( f );

				p.parentNode.removeChild( p );
			}
			
			label.appendChild( file );
			f.appendChild( span1 );
			f.appendChild( span2 );
			f.appendChild( label );
			p.appendChild( f );

			f.containerReference = p;
			
			return p;
		},

		createResultItem: function(result) {
			var text;
			var container = result.container;
			var className;

			L.log(result);

			if (result.response.error) {
				text = result.file.value + ' : ' + result.response.error;
				className = 'error';
			}
			else {
				text = result.response.size;
				className = 'success';
			}

			var span = this._createElement('span', {className: className, innerHTML: text});
			container.appendChild( span );
			
			if (this.watcher && this.watcher.createResultItem)
				this.watcher.createResultItem( span );
		},

		addToFileList: function(filename, title) {
			var idx = filename.lastIndexOf('\\');
			if (idx >= 0)
				filename = filename.substr(idx + 1); 
			
			L.log('AddToFileLIst ' + filename);
			if (this.files) {
				if (this.files[ filename ]) {
					L.log('false');
					return false;
				}
			}
			
			if(!title){
				title = this.strings.UNTITLED;
			}
			
			this.files[ filename ] = true;
			
			L.log(this.files);
			
			var tr = this._createElement('tr');
			tr.appendChild(this._createElement('td', {align: 'left', innerHTML: filename + ' '}));
			
			// add title field
			var td_title = this._createElement('td');
			td_title.appendChild(document.createTextNode(this.strings.TITLE + ': '));
			td_title.appendChild(this._createElement('input', {
				type: 'text',
				size: '50',
				name: this.filename+'[title][]',
				value: title
			}));
			tr.appendChild( td_title );
			var td = this._createElement('td');
			
			var deleteFunction = this.onDeleteFile;
			
			var a = this._createElement('a', {
				href: 'javascript:void(0);',
				innerHTML: this.strings.REMOVE,
				domHelper: this,
				filename: filename,
				'onclick': function() {
					deleteFunction( a );
				}
			});
			
			td.appendChild( a );

			td.appendChild(this._createElement('input', {
				type: 'hidden',
				name: this.filename+'[name][]',
				value: filename
			}));		

			tr.appendChild( td );
			
			this.filelist.appendChild( tr );
			
			return true;
		},
		
		removeFromFileList: function(row, filename) {
			L.log('removeFromFileList: ' + filename);
			L.log(this.files);
			if (this.files[ filename ]) {
				var vals = this.uploader.validators;
				for (var i in vals) {
					if (vals[ i ].incrementFileCount)
						vals[ i ].incrementFileCount( -1 );
				}
				
				if (row) {
					this.files[filename] = false;
					row.parentNode.removeChild( row );
				}
			}
		},
		
		onSubmit: function(form) {
			L.log('onSubmit domHelper');
			var p = form.parentNode;
			
			var e = this._createElement('img', {
				className: 'removeMeNextTime',
				src: this.loadingImg,
				visible: true,
				display: 'block'
			});
			L.log( e );
			p.appendChild( e );

			form.style.display = 'none';
		},
		
		onAfterFileUpload: function(form, frame, data) {
			if (form == undefined || !data) {
				L.log('onAfterFileUpload: form or data is undefined');
				return;
			}

			L.log('onAfterFileUpload');
			this._removeControls(form, frame);
			
			this.createResultItem(data);
			if (!data.response.error) {
				// If the file is on the list:
				if (!this.addToFileList(form.elements[ 0 ].value)) {
					this.removeFromFileList(null, form.elements[ 0 ].value);
				}
			}
		},	

		onUploadFail: function(form, errors) {
			if (errors) {
				var text = '';
				for (var i = 0; i < errors.length; i++) {
					if (errors[ i ].message)
						text += errors[ i ].message + '<br />';
				}

				for (var i = 0; i < errors.length; i++) {
					if (errors[ i ] && errors[ i ].type && errors[ i ].type == 'empty') {
						L.log('onUploadFail empty form');
					}
					else if (errors[ i ] && errors[ i ].type && errors[ i ].type == 'bad_extension') {
						L.log('Bad extension');
						alert( errors[ 0 ].message );
					}
					else {
						form.parentNode.appendChild(this._createElement('span', {
							innerHTML: text,
							className: 'uploader_failed'
						}));
					}
				}
				
				this.uploader.popForm(form);
				var p = form.parentNode;
				p.parentNode.removeChild( p );
			}
		},

		onFileLimit: function( max ) {
			alert(this.strings.FILE_LIMIT_ERROR.replace('%d', max));
		},

		_createElement: function(type, attr) {
			var f = document.createElement(type.toUpperCase());
			for (var i in attr)
				f[ i ] = attr[ i ];
			return f;
		},

		_removeControls: function(form, frame) {
			if (frame && /msie/i.test(navigator.userAgent) && !/opera/i.test(navigator.userAgent)) {
				// remove the iframe with it's container, if we are in IE
				e = document.getElementById(frame.id + '_outer');
				e.parentNode.removeChild( e );
			}

			if (form) {
				try {
					// Empty container:
					var p = form.parentNode;
					p.removeChild( form.nextSibling );
					p.removeChild( form );
				}
				catch ( e ) {
				}
			}
		}
	}

var JS_FileValidator = function(uploader, opts) {
	this.options = opts;
	this.uploader = uploader;
};

JS_FileValidator.prototype = {
	strings: {
		BAD_EXT: 'Extension is not allowed : '
	},
	options: {
		ext: ['jpg', 'gif'],
		max: 5,
		current: 0
	},
	uploader: null,
	
	validateExt: function( input ) {
		L.log('validating extension');
		L.log(this.options.ext);
		L.log(input.value);
		
		if (this.options.ext) {
			var exts = this.options.ext;
			
			var last = input.value.lastIndexOf('.');
			var ext = input.value.substring(last + 1).toLowerCase();
			
			var result = false;
			
			for (var i = 0; i < exts.length; i++) {
				L.log(exts[ i ].toLowerCase() + ' = ' + ext);
				if (exts[ i ].toLowerCase() == ext) {
					result = true;
					break;
				}
			}
			
			if (!result) {
				return {
					message: this.strings.BAD_EXT + ext,
					type: 'bad_extension'
				};
			}
		}
		return true;
	},
	
	validateCount: function(count) {
		L.log('validating count');
		if (this.options.max) {
			L.log('have max: ' + this.getMax()+ ' count ' + count + ' ' + this.options.current);
			return this.getMax() >= count + this.options.current;
		}
		return true;
	},

	onAfterFileUpload: function(form, frame, data) {
		if (form == undefined || !data) {
			return;
		}

		if (!data.response.error) {
			this.incrementFileCount( 1 );
		}
	},

	incrementFileCount: function( d ) {
		this.options.current += d;
	},
	
	getMax: function() {
		if (this.options.max && this.options.max > 0)
			return this.options.max;
			
		return 99999;
	}
}